from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceCollectionABC
from cpl_core.environment import ApplicationEnvironmentABC
from cpl_discord.service.discord_collection_abc import DiscordCollectionABC

from bot_core.abc.module_abc import ModuleABC
from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from bot_data.abc.api_key_repository_abc import ApiKeyRepositoryABC
from bot_data.abc.auth_user_repository_abc import AuthUserRepositoryABC
from bot_data.abc.auto_role_repository_abc import AutoRoleRepositoryABC
from bot_data.abc.client_repository_abc import ClientRepositoryABC
from bot_data.abc.known_user_repository_abc import KnownUserRepositoryABC
from bot_data.abc.level_repository_abc import LevelRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.statistic_repository_abc import StatisticRepositoryABC
from bot_data.abc.user_joined_server_repository_abc import UserJoinedServerRepositoryABC
from bot_data.abc.user_joined_voice_channel_repository_abc import (
    UserJoinedVoiceChannelRepositoryABC,
)
from bot_data.abc.user_message_count_per_hour_repository_abc import (
    UserMessageCountPerHourRepositoryABC,
)
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.service.api_key_repository_service import ApiKeyRepositoryService
from bot_data.service.auth_user_repository_service import AuthUserRepositoryService
from bot_data.service.auto_role_repository_service import AutoRoleRepositoryService
from bot_data.service.client_repository_service import ClientRepositoryService
from bot_data.service.known_user_repository_service import KnownUserRepositoryService
from bot_data.service.level_repository_service import LevelRepositoryService
from bot_data.service.seeder_service import SeederService
from bot_data.service.server_repository_service import ServerRepositoryService
from bot_data.service.statistic_repository_service import StatisticRepositoryService
from bot_data.service.user_joined_server_repository_service import (
    UserJoinedServerRepositoryService,
)
from bot_data.service.user_joined_voice_channel_repository_service import (
    UserJoinedVoiceChannelRepositoryService,
)
from bot_data.service.user_message_count_per_hour_repository_service import (
    UserMessageCountPerHourRepositoryService,
)
from bot_data.service.user_repository_service import UserRepositoryService


class DataModule(ModuleABC):
    def __init__(self, dc: DiscordCollectionABC):
        ModuleABC.__init__(self, dc, FeatureFlagsEnum.data_module)

    def configure_configuration(self, config: ConfigurationABC, env: ApplicationEnvironmentABC):
        pass

    def configure_services(self, services: ServiceCollectionABC, env: ApplicationEnvironmentABC):
        services.add_transient(ApiKeyRepositoryABC, ApiKeyRepositoryService)
        services.add_transient(AuthUserRepositoryABC, AuthUserRepositoryService)
        services.add_transient(ServerRepositoryABC, ServerRepositoryService)
        services.add_transient(UserRepositoryABC, UserRepositoryService)
        services.add_transient(ClientRepositoryABC, ClientRepositoryService)
        services.add_transient(KnownUserRepositoryABC, KnownUserRepositoryService)
        services.add_transient(UserJoinedServerRepositoryABC, UserJoinedServerRepositoryService)
        services.add_transient(UserJoinedVoiceChannelRepositoryABC, UserJoinedVoiceChannelRepositoryService)
        services.add_transient(AutoRoleRepositoryABC, AutoRoleRepositoryService)
        services.add_transient(LevelRepositoryABC, LevelRepositoryService)
        services.add_transient(StatisticRepositoryABC, StatisticRepositoryService)
        services.add_transient(
            UserMessageCountPerHourRepositoryABC,
            UserMessageCountPerHourRepositoryService,
        )

        services.add_transient(SeederService)
