from cpl_core.dependency_injection import ServiceProviderABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_query.extension import List

from bot_data.model.auto_role_rule import AutoRoleRule
from bot_graphql.abc.filter_abc import FilterABC


class AutoRoleRuleFilter(FilterABC):
    def __init__(self, services: ServiceProviderABC, bot: DiscordBotServiceABC):
        FilterABC.__init__(self)
        self._services = services
        self._bot = bot

        self._id = None
        self._emoji_name = None
        self._role_id = None
        self._role_name = None
        self._auto_role = None

    def from_dict(self, values: dict):
        if "id" in values:
            self._id = int(values["id"])

        if "emojiName" in values:
            self._emoji_name = values["emojiName"]

        if "roleId" in values:
            self._role_id = int(values["roleId"])

        if "roleName" in values:
            self._role_name = values["roleName"]

        if "autoRole" in values:
            from bot_graphql.filter.auto_role_filter import AutoRoleFilter

            self._auto_role: AutoRoleFilter = self._services.get_service(AutoRoleFilter)
            self._auto_role.from_dict(values["auto_role"])

    def filter(self, query: List[AutoRoleRule]) -> List[AutoRoleRule]:
        if self._id is not None:
            query = query.where(lambda x: x.auto_role_rule_id == self._id)

        if self._emoji_name is not None:
            query = query.where(lambda x: x.emoji_name == self._emoji_name)

        if self._role_id is not None:
            query = query.where(lambda x: x.role_id == self._role_id)

        if self._role_name is not None and self._role_id is not None:

            def get_role_name(x: AutoRoleRule):
                guild = self._bot.get_guild(x.auto_role.server.discord_server_id)
                name = guild.get_role(x.role_id).name
                return name == self._role_name or self._role_name in name

            query = query.where(get_role_name)

        if self._auto_role is not None:
            auto_roles = self._auto_role.filter(query.select(lambda x: x.auto_role)).select(lambda x: x.auto_role_id)
            query = query.where(lambda x: x.auto_role.auto_role_id in auto_roles)

        return query
