from cpl_core.dependency_injection import ServiceProviderABC
from cpl_query.extension import List

from bot_data.model.level import Level
from bot_graphql.abc.filter_abc import FilterABC


class LevelFilter(FilterABC):
    def __init__(self, services: ServiceProviderABC):
        FilterABC.__init__(self)

        self._services = services

        self._id = None
        self._name = None
        self._server = None

    def from_dict(self, values: dict):
        if "id" in values:
            self._id = int(values["id"])

        if "name" in values:
            self._name = values["name"]

        if "server" in values:
            from bot_graphql.filter.server_filter import ServerFilter

            self._server: ServerFilter = self._services.get_service(LevelFilter)
            self._server.from_dict(values["server"])

    def filter(self, query: List[Level]) -> List[Level]:
        if self._id is not None:
            query = query.where(lambda x: x.id == self._id)

        if self._name is not None:
            query = query.where(lambda x: self._name.lower() == x.name.lower() or self._name.lower() in x.name.lower())

        if self._server is not None:
            servers = self._server.filter(query.select(lambda x: x.server)).select(lambda x: x.server_id)
            query = query.where(lambda x: x.server.server_id in servers)

        return query
