from typing import Optional

from cpl_core.dependency_injection import ServiceProviderABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_query.extension import List

from bot_core.abc.client_utils_abc import ClientUtilsABC
from bot_data.model.user import User
from bot_graphql.abc.filter_abc import FilterABC
from bot_graphql.filter.level_filter import LevelFilter
from modules.level.service.level_service import LevelService


class UserFilter(FilterABC):
    def __init__(
        self,
        services: ServiceProviderABC,
        bot: DiscordBotServiceABC,
        client_utils: ClientUtilsABC,
        levels: LevelService,
    ):
        FilterABC.__init__(self)

        self._services = services
        self._bot = bot
        self._client_utils = client_utils
        self._levels = levels

        self._id = None
        self._discord_id = None
        self._name = None
        self._xp = None
        self._ontime = None
        self._level: Optional[LevelFilter] = None

    def from_dict(self, values: dict):
        if "id" in values:
            self._id = int(values["id"])

        if "discordId" in values:
            self._discord_id = int(values["discordId"])

        if "name" in values:
            self._name = values["name"]

        if "xp" in values:
            self._xp = int(values["xp"])

        if "ontime" in values:
            self._ontime = int(values["ontime"])

        if "level" in values:
            self._level: LevelFilter = self._services.get_service(LevelFilter)
            self._level.from_dict(values["level"])

        if "server" in values:
            from bot_graphql.filter.server_filter import ServerFilter

            self._server: ServerFilter = self._services.get_service(ServerFilter)
            self._server.from_dict(values["server"])

    def filter(self, query: List[User]) -> List[User]:
        if self._id is not None:
            query = query.where(lambda x: x.id == self._id)

        if self._discord_id is not None:
            query = query.where(lambda x: x.discord_id == self._discord_id)

        if self._name is not None:
            query = query.where(
                lambda x: self._bot.get_user(x.discord_id).name == self._name
                or self._name in self._bot.get_user(x.discord_id).name
            )

        if self._xp is not None:
            query = query.where(lambda x: x.xp == self._xp)

        if self._ontime is not None:
            query = query.where(lambda x: self._client_utils.get_ontime_for_user(x) == self._ontime)

        if self._level is not None:
            levels = self._level.filter(query.select(lambda x: self._levels.get_level(x))).select(lambda x: x.id)
            query = query.where(lambda x: self._levels.get_level(x).id in levels)

        if self._server is not None:
            servers = self._server.filter(query.select(lambda x: x.server)).select(lambda x: x.server_id)
            query = query.where(lambda x: x.server.server_id in servers)

        return query
