from cpl_core.dependency_injection import ServiceProviderABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_query.extension import List

from bot_data.model.user_joined_server import UserJoinedServer
from bot_graphql.abc.filter_abc import FilterABC


class UserJoinedServerFilter(FilterABC):
    def __init__(
        self,
        services: ServiceProviderABC,
        bot: DiscordBotServiceABC,
    ):
        FilterABC.__init__(self)

        self._services = services
        self._bot = bot

        self._id = None
        self._user = None
        self._joined_on = None
        self._leaved_on = None

    def from_dict(self, values: dict):
        if "id" in values:
            self._id = int(values["id"])

        if "user" in values:
            from bot_graphql.filter.user_filter import UserFilter

            self._user: UserFilter = self._services.get_service(UserFilter)
            self._user.from_dict(values["user"])

        if "joinedOn" in values:
            self._joined_on = values["joinedOn"]

        if "leavedOn" in values:
            self._leaved_on = values["leavedOn"]

    def filter(self, query: List[UserJoinedServer]) -> List[UserJoinedServer]:
        if self._id is not None:
            query = query.where(lambda x: x.id == self._id)

        if self._user is not None:
            users = self._user.filter(query.select(lambda x: x.user)).select(lambda x: x.user_id)
            query = query.where(lambda x: x.user.user_id in users)

        if self._joined_on is not None:
            query = query.where(lambda x: x.joined_on == self._joined_on or self._joined_on in x.joined_on)

        if self._leaved_on is not None:
            query = query.where(lambda x: x.leaved_on == self._leaved_on or self._leaved_on in x.leaved_on)

        return query
