from cpl_core.database.context import DatabaseContextABC

from bot_data.abc.auto_role_repository_abc import AutoRoleRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.model.auto_role import AutoRole
from bot_data.model.user_role_enum import UserRoleEnum
from bot_graphql.abc.query_abc import QueryABC


class AutoRoleMutation(QueryABC):
    def __init__(
        self,
        servers: ServerRepositoryABC,
        auto_roles: AutoRoleRepositoryABC,
        db: DatabaseContextABC,
    ):
        QueryABC.__init__(self, "AutoRoleMutation")

        self._servers = servers
        self._auto_roles = auto_roles
        self._db = db

        self.set_field("createAutoRole", self.resolve_create_auto_role)
        self.set_field("updateAutoRole", self.resolve_update_auto_role)
        self.set_field("deleteAutoRole", self.resolve_delete_auto_role)

    def resolve_create_auto_role(self, *_, input: dict):
        auto_role = AutoRole(self._servers.get_server_by_id(input["serverId"]), input["channelId"], input["messageId"])
        self._can_user_mutate_data(auto_role.server, UserRoleEnum.moderator)

        self._auto_roles.add_auto_role(auto_role)
        self._db.save_changes()

        def get_new(x: AutoRole):
            return (
                x.server.server_id == input["serverId"]
                and x.discord_channel_id == input["channelId"]
                and x.discord_message_id == input["messageId"]
            )

        return self._auto_roles.get_auto_roles_by_server_id(auto_role.server.server_id).where(get_new).last()

    def resolve_update_auto_role(self, *_, input: dict):
        auto_role = self._auto_roles.get_auto_role_by_id(input["id"])
        self._can_user_mutate_data(auto_role.server, UserRoleEnum.moderator)

        auto_role.discord_channel_id = input["channelId"] if "channelId" in input else auto_role.discord_channel_id
        auto_role.discord_message_id = input["messageId"] if "messageId" in input else auto_role.discord_message_id

        self._auto_roles.update_auto_role(auto_role)
        self._db.save_changes()

        auto_role = self._auto_roles.get_auto_role_by_id(input["id"])
        return auto_role

    def resolve_delete_auto_role(self, *_, id: int):
        auto_role = self._auto_roles.get_auto_role_by_id(id)
        self._can_user_mutate_data(auto_role.server, UserRoleEnum.moderator)

        self._auto_roles.delete_auto_role(auto_role)
        self._db.save_changes()
        return auto_role
