from cpl_core.database.context import DatabaseContextABC

from bot_data.abc.level_repository_abc import LevelRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.model.level import Level
from bot_data.model.user_role_enum import UserRoleEnum
from bot_graphql.abc.query_abc import QueryABC


class LevelMutation(QueryABC):
    def __init__(
        self,
        servers: ServerRepositoryABC,
        levels: LevelRepositoryABC,
        db: DatabaseContextABC,
    ):
        QueryABC.__init__(self, "LevelMutation")

        self._servers = servers
        self._levels = levels
        self._db = db

        self.set_field("createLevel", self.resolve_create_level)
        self.set_field("updateLevel", self.resolve_update_level)
        self.set_field("deleteLevel", self.resolve_delete_level)

    def resolve_create_level(self, *_, input: dict):
        server = self._servers.get_server_by_id(input["serverId"])
        self._can_user_mutate_data(server, UserRoleEnum.admin)

        level = Level(
            input["name"],
            input["color"],
            int(input["minXp"]),
            int(input["permissions"]),
            server,
        )
        self._levels.add_level(level)
        self._db.save_changes()

        def get_new_level(l: Level):
            return (
                l.name == level.name
                and l.color == level.color
                and l.min_xp == level.min_xp
                and l.permissions == level.permissions
            )

        return self._levels.get_levels_by_server_id(level.server.server_id).where(get_new_level).last()

    def resolve_update_level(self, *_, input: dict):
        level = self._levels.get_level_by_id(input["id"])
        self._can_user_mutate_data(level.server, UserRoleEnum.admin)

        level.name = input["name"] if "name" in input else level.name
        level.color = input["color"] if "color" in input else level.color
        level.min_xp = input["minXp"] if "minXp" in input else level.min_xp
        level.permissions = input["permissions"] if "permissions" in input else level.permissions

        self._levels.update_level(level)
        self._db.save_changes()

        level = self._levels.get_level_by_id(input["id"])
        return level

    def resolve_delete_level(self, *_, id: int):
        level = self._levels.get_level_by_id(id)
        self._can_user_mutate_data(level.server, UserRoleEnum.admin)

        self._levels.delete_level(level)
        self._db.save_changes()
        return level
