from cpl_core.database.context import DatabaseContextABC
from cpl_discord.service import DiscordBotServiceABC

from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.user_role_enum import UserRoleEnum
from bot_graphql.abc.query_abc import QueryABC
from modules.permission.service.permission_service import PermissionService


class UserMutation(QueryABC):
    def __init__(
        self,
        servers: ServerRepositoryABC,
        users: UserRepositoryABC,
        bot: DiscordBotServiceABC,
        db: DatabaseContextABC,
        permissions: PermissionService,
    ):
        QueryABC.__init__(self, "UserMutation")

        self._servers = servers
        self._users = users
        self._bot = bot
        self._db = db
        self._permissions = permissions

        self.set_field("updateUser", self.resolve_update_user)

    def resolve_update_user(self, *_, input: dict):
        user = self._users.get_user_by_id(input["id"])
        self._can_user_mutate_data(user.server, UserRoleEnum.moderator)

        user.xp = input["xp"] if "xp" in input else user.xp

        self._users.update_user(user)
        self._db.save_changes()

        user = self._users.get_user_by_id(input["id"])
        return user
