from cpl_discord.service import DiscordBotServiceABC

from bot_data.abc.auto_role_repository_abc import AutoRoleRepositoryABC
from bot_data.abc.client_repository_abc import ClientRepositoryABC
from bot_data.abc.level_repository_abc import LevelRepositoryABC
from bot_data.abc.user_joined_server_repository_abc import UserJoinedServerRepositoryABC
from bot_data.abc.user_joined_voice_channel_repository_abc import UserJoinedVoiceChannelRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.server import Server
from bot_graphql.abc.data_query_abc import DataQueryABC
from bot_graphql.filter.auto_role_filter import AutoRoleFilter
from bot_graphql.filter.client_filter import ClientFilter
from bot_graphql.filter.level_filter import LevelFilter
from bot_graphql.filter.user_filter import UserFilter


class ServerQuery(DataQueryABC):
    def __init__(
        self,
        bot: DiscordBotServiceABC,
        auto_roles: AutoRoleRepositoryABC,
        clients: ClientRepositoryABC,
        levels: LevelRepositoryABC,
        users: UserRepositoryABC,
        ujs: UserJoinedServerRepositoryABC,
        ujvs: UserJoinedVoiceChannelRepositoryABC,
    ):
        DataQueryABC.__init__(self, "Server")

        self._bot = bot
        self._auto_roles = auto_roles
        self._clients = clients
        self._levels = levels
        self._users = users
        self._ujs = ujs
        self._ujvs = ujvs

        self.set_field("id", self.resolve_id)
        self.set_field("discordId", self.resolve_discord_id)
        self.set_field("name", self.resolve_name)
        self.set_field("iconURL", self.resolve_icon_url)

        self.add_collection(
            "autoRole",
            lambda server, *_: self._auto_roles.get_auto_roles_by_server_id(server.server_id),
            AutoRoleFilter,
        )
        self.add_collection(
            "client", lambda server, *_: self._clients.get_clients_by_server_id(server.server_id), ClientFilter
        )
        self.add_collection(
            "level", lambda server, *_: self._levels.get_levels_by_server_id(server.server_id), LevelFilter
        )
        self.add_collection("user", lambda server, *_: self._users.get_users_by_server_id(server.server_id), UserFilter)

    @staticmethod
    def resolve_id(server: Server, *_):
        return server.server_id

    @staticmethod
    def resolve_discord_id(server: Server, *_):
        return server.discord_server_id

    def resolve_name(self, server: Server, *_):
        guild = self._bot.get_guild(server.discord_server_id)
        return None if guild is None else guild.name

    def resolve_icon_url(self, server: Server, *_):
        guild = self._bot.get_guild(server.discord_server_id)
        return None if guild is None else guild.icon.url
