from cpl_discord.service import DiscordBotServiceABC

from bot_core.abc.client_utils_abc import ClientUtilsABC
from bot_data.abc.user_joined_server_repository_abc import UserJoinedServerRepositoryABC
from bot_data.abc.user_joined_voice_channel_repository_abc import UserJoinedVoiceChannelRepositoryABC
from bot_data.model.user import User
from bot_graphql.abc.data_query_abc import DataQueryABC
from bot_graphql.filter.user_joined_server_filter import UserJoinedServerFilter
from bot_graphql.filter.user_joined_voice_channel_filter import UserJoinedVoiceChannelFilter
from modules.level.service.level_service import LevelService


class UserQuery(DataQueryABC):
    def __init__(
        self,
        bot: DiscordBotServiceABC,
        levels: LevelService,
        client_utils: ClientUtilsABC,
        ujs: UserJoinedServerRepositoryABC,
        ujvs: UserJoinedVoiceChannelRepositoryABC,
    ):
        DataQueryABC.__init__(self, "User")

        self._bot = bot
        self._levels = levels
        self._client_utils = client_utils
        self._ujs = ujs
        self._ujvs = ujvs

        self.set_field("id", self.resolve_id)
        self.set_field("discordId", self.resolve_discord_id)
        self.set_field("name", self.resolve_name)
        self.set_field("xp", self.resolve_xp)
        self.set_field("ontime", self.resolve_ontime)
        self.set_field("level", self.resolve_level)
        self.add_collection(
            "joinedServer",
            lambda user, *_: self._ujs.get_user_joined_servers_by_user_id(user.user_id),
            UserJoinedServerFilter,
        )
        self.add_collection(
            "joinedVoiceChannel",
            lambda user, *_: self._ujvs.get_user_joined_voice_channels_by_user_id(user.user_id),
            UserJoinedVoiceChannelFilter,
        )
        self.set_field("server", self.resolve_server)

    @staticmethod
    def resolve_id(user: User, *_):
        return user.user_id

    @staticmethod
    def resolve_discord_id(user: User, *_):
        return user.discord_id

    def resolve_name(self, user: User, *_):
        guild = self._bot.get_guild(user.server.discord_server_id)
        user = guild.get_member(user.discord_id)
        return None if user is None else user.name

    @staticmethod
    def resolve_xp(user: User, *_):
        return user.xp

    def resolve_ontime(self, user: User, *_):
        return self._client_utils.get_ontime_for_user(user)

    def resolve_level(self, user: User, *_):
        return self._levels.get_level(user)

    @staticmethod
    def resolve_server(user: User, *_):
        return user.server
