import { Component, OnInit } from "@angular/core";
import { LangChangeEvent, TranslateService } from "@ngx-translate/core";
import { MenuItem } from "primeng/api";
import { AuthService } from "src/app/services/auth/auth.service";
import { ThemeService } from "src/app/services/theme/theme.service";
import { SidebarService } from "../../services/sidebar/sidebar.service";

@Component({
  selector: "app-sidebar",
  templateUrl: "./sidebar.component.html",
  styleUrls: ["./sidebar.component.scss"]
})
export class SidebarComponent implements OnInit {

  isSidebarOpen!: boolean;
  menuItems!: MenuItem[];

  constructor(
    private authService: AuthService,
    private translateService: TranslateService,
    private themeService: ThemeService,
    private sidebar: SidebarService
  ) {
    this.translateService.onLangChange.subscribe((event: LangChangeEvent) => {
      this.sidebar.setMenu();
    });

    this.themeService.isSidebarOpen$.subscribe(value => {
      this.isSidebarOpen = value;
      this.sidebar.setMenu();
    });

    this.sidebar.menuItems$.subscribe(value => {
      this.menuItems = value;
    });
  }

  ngOnInit(): void {
    this.themeService.loadMenu();
  }

}
