import { Component, OnInit } from "@angular/core";
import { ActivatedRoute, Router } from "@angular/router";
import { Server } from "src/app/models/data/server.model";
import { DataService } from "src/app/services/data/data.service";
import { SpinnerService } from "src/app/services/spinner/spinner.service";
import { Queries } from "../../../../models/graphql/queries.model";
import { Query } from "../../../../models/graphql/query.model";
import { SidebarService } from "../../../../services/sidebar/sidebar.service";

@Component({
  selector: "app-server-dashboard",
  templateUrl: "./server-dashboard.component.html",
  styleUrls: ["./server-dashboard.component.scss"]
})
export class ServerDashboardComponent implements OnInit {

  id!: number;
  server!: Server;

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private data: DataService,
    private spinner: SpinnerService,
    private sidebar: SidebarService
  ) {
  }

  ngOnInit(): void {
    this.spinner.showSpinner();
    if (!this.route.snapshot.params["serverId"]) {
      this.spinner.hideSpinner();
      this.router.navigate(['/dashboard']);
      return;
    }

    this.data.query<Server>(Queries.serversQuery, {
        filter: { id: this.route.snapshot.params["serverId"] }
      },
      function(data: Query) {
        return data.servers.length > 0 ? data.servers[0] : null;
      }
    ).subscribe(server => {
      this.server = server;
      this.sidebar.serverName$.next(server.name ?? "");
      this.spinner.hideSpinner();
    });
  }

}
